/*****************************************************
CVAVR V2.05
Chip type               : ATmega8
AVR Core Clock frequency: 8 MHz
Project: st_gen
*****************************************************/
#include <mega8.h>
#include <math.h>

#define A       PORTB.6
#define B       PORTD.4
#define C       PORTD.3
#define D       PORTD.2
#define E       PORTD.1
#define F       PORTD.0
#define G       PORTC.5
#define DP      PORTC.4
#define N0      PORTD.6
#define N1      PORTD.5
#define N2      PORTB.7

#define key0    PINB.5

#define HL      PORTB.3
#define HR      PORTB.4

typedef unsigned char u8;
typedef unsigned int u16;
typedef unsigned long u32;

#define Rb_upr 5100 //Ohms
#define Rb_lwr 1000   //Ohms
#define Ub_max_adc ((256UL*(Rb_upr+Rb_lwr))/Rb_lwr) //=U_max*100, 15,6V*100
#define Rhv_upr 680000 //Ohms
#define Rhv_lwr 5100   //Ohms  
#define UHV_max_adc ((256UL*(Rhv_upr+Rhv_lwr))/Rhv_lwr) //=U_max*100, 343,9V*100

#define cpu_freq 8000000UL //Hz
#define cnt_res 0xff
#define pwm_freq  cpu_freq/(2*cnt_res)
#define base_freq 50 //Hz

#define step pwm_freq/(base_freq*2)
#define pwm_top 0xff

#define pwm_off do{OCR1AL=0;OCR1BL=pwm_top;a=0;aco_cnt=step;}while(0)

enum btn_states {prs, rls, hold} btn0;
enum dmodes {run,opr,lobt,ocp,sdn} dev_st=run;

#define timnmbr 3
u8 tkey, tmr[timnmbr];
u16 tim[timnmbr];

u16 adc1val, adc0val;

u8 dbuf[3]={0,0,2};
u8 dpp=0;

u8 s_table [step/2];
u8 aco_cnt;

u8 a=0xff;
bit cntfb, r_adc1=0, r_adc0=0;

void gpio_cfg()
{
DDRB=0b11011111; PORTB.5=PORTB.0=1;
DDRC=0b11111100;
DDRD=0b01111111; PORTD.7=1;
}
void t1_cfg()
{
 TCCR1A = 1<<COM1A1|1<<COM1B1|1<<COM1A0|1<<WGM10;
 TCCR1B = 1;
 OCR1A  = 0;
 OCR1B  = 0xff;
 TIMSK  = 1<<TOIE1;
}
void acmp_cfg()
{
 ACSR = 1<<ACBG|1<<ACIS1|1<<ACIS0;
 ACSR|= 1<<ACIE;
}
void adc_cfg()
{
 ADMUX=1<<REFS1|1<<REFS0;
 ADCSRA=1<<ADEN|4;
 ADCSRA|=1<<ADSC;
}

void convert_data (u16 d)
{
u8 k;
  for (k=0;k<3;k++)
  {
   dbuf[k]=d%10;
   d/=10;
  }
}
void display_data ()
{
  static u8 pc;

  (pc)?(pc--):(pc = 2);
  N0=N1=N2=0; DP=1;

  switch (dbuf[pc])
 {
  case 0: A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 1: A=1; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 2: A=0; B=0; C=1; D=0; E=0; F=1; G=0; break;
  case 3: A=0; B=0; C=0; D=0; E=1; F=1; G=0; break;
  case 4: A=1; B=0; C=0; D=1; E=1; F=0; G=0; break;
  case 5: A=0; B=1; C=0; D=0; E=1; F=0; G=0; break;
  case 6: A=0; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 7: A=0; B=0; C=0; D=1; E=1; F=1; G=1; break;
  case 8: A=0; B=0; C=0; D=0; E=0; F=0; G=0; break;
  case 9: A=0; B=0; C=0; D=0; E=1; F=0; G=0; break;
  case ' ': A=1; B=1; C=1; D=1; E=1; F=1; G=1; break;
  case 'u': A=1; B=1; C=0; D=0; E=0; F=1; G=1; break;
  case 'c': A=1; B=1; C=1; D=0; E=0; F=1; G=0; break;
  case 'b': A=1; B=1; C=0; D=0; E=0; F=0; G=0; break;
  case 't': A=1; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'l': A=1; B=1; C=1; D=0; E=0; F=1; G=1; break;
  case 'o': A=1; B=1; C=0; D=0; E=0; F=1; G=0; break;
  case 'A': A=0; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'H': A=1; B=0; C=0; D=1; E=0; F=0; G=0; break;
  case 'C': A=0; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'P': A=0; B=0; C=1; D=1; E=0; F=0; G=0; break;
  case 'E': A=0; B=1; C=1; D=0; E=0; F=0; G=0; break;
  case 'U': A=1; B=0; C=0; D=0; E=0; F=0; G=1; break;
  case 'F': A=0; B=1; C=1; D=1; E=0; F=0; G=0; break;
  case 'L': A=1; B=1; C=1; D=0; E=0; F=0; G=1; break;
  case 'r': A=1; B=1; C=1; D=1; E=0; F=1; G=0; break;
  case '_': A=1; B=1; C=1; D=0; E=1; F=1; G=1; break;
  case 'd': A=1; B=0; C=0; D=0; E=0; F=1; G=0; break;
  case 'n': A=1; B=1; C=0; D=1; E=0; F=1; G=0; break;
  case '': A=0; B=0; C=1; D=1; E=1; F=0; G=1; break;
  case 'O': A=0; B=0; C=0; D=0; E=0; F=0; G=1; break;  
  default:  A=1; B=1; C=1; D=1; E=1; F=1; G=0; break; //-
 }

   switch (pc)
 {
  case 0: N0=1; if(dpp == 0) {DP=0;} break;
  case 1: N1=1; if(dpp == 1) {DP=0;} break;
  case 2: if(dbuf[2]){N2=1;} if(dpp == 2) {DP=0;} break;
  default: DP=1; break;
 }
 }

void scan_keys ()
{
#define lp  16
#define lh  40
#define lr  4

static u8 count[2];

if ((!key0)&&(!tkey)) {count[0]++;tkey=lr;}
  else if((key0)&&(!tkey)) {count[0]=count[1]=0;btn0=rls;}
if ((count[0]>lp)&&(btn0!=hold)) {btn0=prs; count[0]=0; count[1]++;}
if (count[1]>lh) {btn0=hold; count[1]=0;}
}

interrupt [TIM1_OVF] void t1_ovf()
{
static u16 acnt1; 
static u16 acnt0; 
static u32 adc0tmp;
static u32 adc1tmp;
#define dtv 2
static bit d=1;
static u8 mst=0xff, c=0;
u8 tmp_table, i;

if(ADMUX==0xC1) {acnt1++; adc1tmp+=ADCW;} //hv
else {acnt0++; adc0tmp+=ADCW; ADMUX=0xC1;} //bat

if(aco_cnt){if(!ACSR.ACO){aco_cnt--;} else pwm_off;}
else
{
tmp_table=((u16)s_table [c]*mst)>>9;
if(tmp_table>0xff/2){tmp_table=0xff/2;}

if(d)
 {
    if(tmp_table<pwm_top/2-dtv){OCR1AL=tmp_table;OCR1BL=pwm_top-tmp_table;}
    else                       {OCR1AL=pwm_top/2-dtv;OCR1BL=pwm_top/2+dtv;}
  
  if(c<step/2-1){c++;} else {d=0;}
 }
else
  {
    if(tmp_table<pwm_top/2-dtv){OCR1AL=tmp_table;OCR1BL=pwm_top-tmp_table;}
    else                       {OCR1AL=pwm_top/2-dtv;OCR1BL=pwm_top/2+dtv;}

    if(c)c--;
    else
    {
      d=1;
      if(!HL){HL=1;HR=0; mst=a;cntfb=1;}
      else   {HL=0;HR=1;} 

      ADMUX=0xC0;  
      if(!r_adc1){adc1val=adc1tmp/acnt1; adc1tmp=acnt1=0;} 
      if((!r_adc0)&&(acnt0>4)){adc0val=adc0tmp/acnt0; adc0tmp=acnt0=0;}        
    } 
  }
}

ADCSRA.ADSC=1;

 for(i=0;i<timnmbr;i++)
 {
  if(tim[i])tim[i]--;
  if(tmr[i])tmr[i]--;
 }
  if(tkey)tkey--;
#asm("wdr")
}
interrupt [ANA_COMP] void acmp_isr()
{
  pwm_off;
  aco_cnt=step;  
}

void devcon(); void st_hv();
u8 bat_ok();

void main()
{
 u8 i;
 
 gpio_cfg();
 t1_cfg();
 acmp_cfg();
 adc_cfg();

for(i=0;i<step/2;i++){s_table[i]=(float)pwm_top*sin(((PI/2)*i)/(step/2-1));}

#pragma optsize-
WDTCR=0x18;
WDTCR=0x08;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif
#asm("sei")  
while (1)
      {
        display_data(); 
        scan_keys();
        devcon(); 

      }
}

void devcon()
{
  static bit itr=0,btr;  
  
  if(btn0==rls)btr=0;


  switch(dev_st) 
  {
   case run:
   {
    u8 tmp;
    if(!itr)
    {
     itr=1; tim[0]=15625; //1s
     dbuf[2]='r';dbuf[1]='u';dbuf[0]='n';dpp=5;
     pwm_off;
    }
     
     if(!tim[0])
     {
      if(!tmr[0])
      {
       tmr[0]=0xff;r_adc0=1;tmp=((u32)Ub_max_adc*adc0val)/(0x3ffUL*10);r_adc0=0;
       if(tmp>110){dev_st=opr;} else {convert_data(tmp);dpp=1;}
      }
     }
   if(dev_st!=run){itr=0;}
   } break;

   case opr:
   {
    //if(!itr){itr=1; tim[0]=15625;}
    if(!aco_cnt)st_hv();
    if((btn0==hold)&&(btr==0)) {btr=1; dev_st=sdn;}
    if(!tmr[0])
    {
     tmr[0]=0xff; 
     if(!bat_ok()){dev_st=lobt;} 
     else if(!tim[1])
     {tim[1]=0xffff/4;r_adc0=1;convert_data(((u32)Ub_max_adc*adc0val)/(0x3ffUL*10));r_adc0=0;dpp=1;}
    }
    if(dev_st!=opr){itr=0;}
   } break;

   case ocp:
   {
    if(!itr)
    {
     itr=1; tim[0]=15625; //1s
     dbuf[2]='O';dbuf[1]='C';dbuf[0]='P';dpp=5;
     pwm_off;
    }


    if(dev_st!=ocp){itr=0;}
   } break;

   case lobt:
   {
    if(!itr)
    {
     itr=1;
     dbuf[2]='b';dbuf[1]='L';dbuf[0]='o';dpp=2;
     pwm_off;
    }
    
    if((btn0==hold)&&(btr==0)) {btr=1; dev_st=run;}
    if(dev_st!=lobt){itr=0;}
   } break;

   case sdn:
   {
    if(!itr)
    {
     itr=1;
     dbuf[2]='_';dbuf[1]='_';dbuf[0]='_';dpp=5;
     pwm_off;
    }
    
    if((btn0==hold)&&(btr==0)) {btr=1; dev_st=run;}
    if(dev_st!=sdn){itr=0;}
   } break;

   default:break;
  }
 }


void st_hv()
{
 u16 tmpU;

 if(cntfb)
 {
  tmpU = ((u32)UHV_max_adc*adc1val)/(0x3ffUL*100);

	if((tmpU>230)&&(tmpU<240))	{tmr[2]=0xff;if(a){a--;}}
	else if((tmpU>200)&&(tmpU<210)){tmr[2]=0xff;if(a<0xff){a++;}}
	else if(tmpU>=240){if(a>5){a-=5;tmr[2]=0xff;}else a=0;}
	else if((tmpU<=200)&&(a<0xff)){if(a<0xff-5){a+=5;tmr[2]=0xff;}else a=0xff;}
	//else if (tmpU<200){if(a<0xff){a++;}else {a=0xff;}}
	
	//if(!tmr[2]){d_mode=ovl;} else if(a<100) {d_mode=normal;}

 cntfb=0;
 }
}

u8 bat_ok()
{
 u8 tmp;
 static u8 trg0;
 
 r_adc0=1;
 tmp = ((u32)Ub_max_adc*adc0val)/(0x3ffUL*10);
 r_adc0=0;	 
 if(tmp>108){tim[2]=0xffff;}   
 //else if(tmp<100){d_mode=lobat; trg0=1;}
 
 if(tmp>120) 	trg0=0; 
 if(!tim[2])	trg0=1;
 
 if (!trg0)	return 1; else return 0;
}